#include "subtest.h"
#include <vector>

class TestTutorials : public SubTest
{
 public:

  TestTutorials() {
    testSuiteName = "TestTutorials";
    tests.push_back(test0D);
    tests.push_back(test1D);
    tests.push_back(test2D);
    tests.push_back(test3D);
  }

  static TestResult test0D() {
    int level_Info_ = Info.level;
    Info.level = 0;
    std::string suiteName = "Tutorials";
    std::string testName = "Tutorials: Test 1 - Regression testing (0D)";
    std::string testDescription = "";
    const fileName pato_dir = getEnv("PATO_DIR");
    const fileName pato_unit_testing = getEnv("PATO_UNIT_TESTING");
    fileName tutoFolder = pato_dir+"/tutorials/0D";
    fileName refFolder = pato_unit_testing+"/testsuites/tutorials/ref/0D";
    TestResult result(runTestTutorial(testName,testDescription,tutoFolder,refFolder));
    Info.level=level_Info_;
    return result;
  }

  static TestResult test1D() {
    int level_Info_ = Info.level;
    Info.level = 0;
    std::string suiteName = "Tutorials";
    std::string testName = "Tutorials: Test 2 - Regression testing (1D)";
    std::string testDescription = "";
    const fileName pato_dir = getEnv("PATO_DIR");
    const fileName pato_unit_testing = getEnv("PATO_UNIT_TESTING");
    fileName tutoFolder = pato_dir+"/tutorials/1D";
    fileName refFolder = pato_unit_testing+"/testsuites/tutorials/ref/1D";
    TestResult result(runTestTutorial(testName,testDescription,tutoFolder,refFolder));
    Info.level=level_Info_;
    return result;
  }

  static TestResult test2D() {
    int level_Info_ = Info.level;
    Info.level = 0;
    std::string suiteName = "Tutorials";
    std::string testName = "Tutorials: Test 2 - Regression testing (2D)";
    std::string testDescription = "";
    const fileName pato_dir = getEnv("PATO_DIR");
    const fileName pato_unit_testing = getEnv("PATO_UNIT_TESTING");
    fileName tutoFolder = pato_dir+"/tutorials/2D";
    fileName refFolder = pato_unit_testing+"/testsuites/tutorials/ref/2D";
    TestResult result(runTestTutorial(testName,testDescription,tutoFolder,refFolder));
    Info.level=level_Info_;
    return result;
  }

  static TestResult test3D() {
    int level_Info_ = Info.level;
    Info.level = 0;
    std::string suiteName = "Tutorials";
    std::string testName = "Tutorials: Test 3 - Regression testing (3D)";
    std::string testDescription = "";
    const fileName pato_dir = getEnv("PATO_DIR");
    const fileName pato_unit_testing = getEnv("PATO_UNIT_TESTING");
    fileName tutoFolder = pato_dir+"/tutorials/3D";
    fileName refFolder = pato_unit_testing+"/testsuites/tutorials/ref/3D";
    TestResult result(runTestTutorial(testName,testDescription,tutoFolder,refFolder));
    Info.level = level_Info_;
    return result;
  }

  static TestResult runTestTutorial(std::string testName, std::string testDescription, fileName tutoFolder, fileName refFolder) {
    std::string suiteName = "Tutorials";
    TestResult result(suiteName, testName, 1, testDescription);
    fileNameList tutoFolders = searchFoldersKeyword(tutoFolder,"Allrun");
    fileNameList refFolders  = searchFoldersKeyword(refFolder,"Allrun");

    labelList visitOrder;
    sortedOrder(tutoFolders, visitOrder);
    tutoFolders = fileNameList(tutoFolders, visitOrder);
    sortedOrder(refFolders, visitOrder);
    refFolders = fileNameList(refFolders, visitOrder);

    if(!assertEquals(refFolders.size(),tutoFolders.size(), &result)) {

      result.expected="refFolders.size() = " + name(refFolders.size());
      forAll(refFolders, i) {
        result.expected+="\n\t - " + refFolders[i];
      }
      result.expected+="\n";
      result.actual="tutoFolders.size() = " + name(tutoFolders.size());
      forAll(tutoFolders, i) {
        result.actual+="\n\t - " + tutoFolders[i];
      }
      result.actual+="\n";
      return result;
    }

    forAll(tutoFolders,folderI) {
      word folder = (word) tutoFolders[folderI];
      word reffolder = (word) refFolders[folderI];
      const fileName pato_tuto =  getEnv("PATO_TUTORIALS");
      Info << "\t" << testName << " : " << tutoFolders[folderI] << endl;
      system(folder+"/Allclean > log");
      system(folder+"/Allrun > log");

      fileNameList tutoFiles=filesInFolder(folder+"/output");
      fileNameList refFiles=filesInFolder(reffolder+"/output");

      labelList visitOrder;
      sortedOrder(tutoFiles, visitOrder);
      tutoFiles = fileNameList(tutoFiles, visitOrder);
      sortedOrder(refFiles, visitOrder);
      refFiles = fileNameList(refFiles, visitOrder);

      if(!assertEquals(refFiles.size(), tutoFiles.size(), &result)) {
        result.expected="refFiles.size() = " + name(refFiles.size());
        forAll(refFiles, i) {
          result.expected+="\n\t - " + refFiles[i];
        }
        result.expected+="\n";
        result.actual="tutoFiles.size() = " + name(tutoFiles.size());
        forAll(tutoFiles, i) {
          result.actual+="\n\t - " + tutoFiles[i];
        }
        result.actual+="\n";
        return result;
      }

      forAll(tutoFiles, fileI) {
        fileName tutofile = tutoFiles[fileI];
        fileName reffile = refFiles[fileI];
        if(! assertEquals(tutofile,reffile,&result)) {
          return result;
        }
      }
      system(folder+"/Allclean > log");

    }
    system("rm log");
    return result;
  }
};
